@@ -5,7 +5,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -124,6 +124,7 @@ ConfigDSN (
     }
 
   /* Retrieve the corresponding driver */
+#if !defined (__APPLE__)
   if (strstr (lpszDriver, "OpenLink") || strstr (lpszDriver, "Openlink")
       || strstr (lpszDriver, "oplodbc"))
     {
@@ -167,6 +168,7 @@ ConfigDSN (
   else if ((strstr (lpszDriver, "Virtuoso")
 	  || strstr (lpszDriver, "virtodbc")))
     driver_type = 1;
+#endif
 
   /* For each request */
   switch (fRequest)
@@ -250,23 +252,20 @@ ConfigDSN (
       if (connstr == (LPSTR) - 1L)
 	goto done;
 
-      /* Compare if the DSN changed */
-      if (strcmp (connstr + STRLEN ("DSN="), dsn))
-	{
-	  /* Remove the previous DSN */
-	  SQLSetConfigMode (confMode);
-	  if (!SQLRemoveDSNFromIni (dsn))
-	    goto done;
-	  /* Add the new DSN section */
-	  SQLSetConfigMode (confMode);
-	  if (!SQLWriteDSNToIni (dsn = connstr + STRLEN ("DSN="), lpszDriver))
-	    goto done;
-	}
+
+      /* Remove the previous DSN */
+      SQLSetConfigMode (confMode);
+      if (!SQLRemoveDSNFromIni (dsn))
+        goto done;
+
+      /* Add the new DSN section */
+      SQLSetConfigMode (confMode);
+      if (!SQLWriteDSNToIni (dsn = connstr + STRLEN ("DSN="), lpszDriver))
+        goto done;
 
       /* Add each keyword and values */
       for (curr = connstr; *curr; curr += (STRLEN (curr) + 1))
 	{
-
 	  if (strncmp (curr, "DSN=", STRLEN ("DSN=")))
 	    {
 	      STRCPY (dsnread, curr);
