@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -78,8 +78,8 @@
 #include "gui.h"
 
 #include <iodbc.h>
-#include <herr.h>
-#include <dlproc.h>
+#include "herr.h"
+#include "dlproc.h"
 
 
 SQLRETURN SQL_API
@@ -163,7 +163,7 @@ _iodbcdm_drvconn_dialbox (
       *curr = '\0';
     }
 
-  retcode = log_t.ok ? SQL_SUCCESS : SQL_NO_DATA_FOUND;
+  retcode = log_t.ok == TRUE ? SQL_SUCCESS : SQL_NO_DATA_FOUND;
 
 quit:
   for (curr = szInOutConnStr; *curr; curr = szDSN + 1)
