@@ -1,12 +1,78 @@
 /*
  *  iodbcext.h
  *
- *  $Id: iodbcext.h,v 1.4 2004/07/06 15:26:06 source Exp $
+ *  $Id$
  *
  *  iODBC extensions and changes to the SAG specifications
  *  for third party drivers
  *
- *  Driver specific constants.
+ *  The iODBC driver manager.
+ *
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
+ *  All Rights Reserved.
+ *
+ *  This software is released under the terms of either of the following
+ *  licenses:
+ *
+ *      - GNU Library General Public License (see LICENSE.LGPL)
+ *      - The BSD License (see LICENSE.BSD).
+ *
+ *  Note that the only valid version of the LGPL license as far as this
+ *  project is concerned is the original GNU Library General Public License
+ *  Version 2, dated June 1991.
+ *
+ *  While not mandated by the BSD license, any patches you make to the
+ *  iODBC source code may be contributed back into the iODBC project
+ *  at your discretion. Contributions will benefit the Open Source and
+ *  Data Access community as a whole. Submissions may be made at:
+ *
+ *      http://www.iodbc.org
+ *
+ *
+ *  GNU Library Generic Public License Version 2
+ *  ============================================
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; only
+ *  Version 2 of the License dated June 1991.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the Free
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ *
+ *  The BSD License
+ *  ===============
+ *  Redistribution and use in source and binary forms, with or without
+ *  modification, are permitted provided that the following conditions
+ *  are met:
+ *
+ *  1. Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions and the following disclaimer.
+ *  2. Redistributions in binary form must reproduce the above copyright
+ *     notice, this list of conditions and the following disclaimer in
+ *     the documentation and/or other materials provided with the
+ *     distribution.
+ *  3. Neither the name of OpenLink Software Inc. nor the names of its
+ *     contributors may be used to endorse or promote products derived
+ *     from this software without specific prior written permission.
+ *
+ *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OPENLINK OR
+ *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
@@ -144,6 +210,32 @@
 #define SQL_RE_DEFAULT	SQL_RE_OFF
 
 
+#define SQL_ATTR_APP_WCHAR_TYPE		      1061
+
+#define SQL_ATTR_APP_UNICODE_TYPE             1064
+#define SQL_ATTR_DRIVER_UNICODE_TYPE          1065
+
+#define SQL_DM_CP_UTF16			1
+#define SQL_DM_CP_UTF8			2
+#define SQL_DM_CP_UCS4			3
+
+#if defined(SIZEOF_WCHAR)
+# if (SIZEOF_WCHAR == 2)
+#  define SQL_DM_CP_DEF  SQL_DM_CP_UTF16
+# else
+#  define SQL_DM_CP_DEF  SQL_DM_CP_UCS4
+# endif
+#else
+# define SQL_DM_CP_DEF  SQL_DM_CP_UCS4
+#endif
+
+/* DataDirect options for Connection/Env Attributes. */
+#define SQL_DD_CP_ANSI				0 
+#define SQL_DD_CP_UCS2				SQL_DM_CP_UTF16
+#define SQL_DD_CP_UTF8				SQL_DM_CP_UTF8
+#define SQL_DD_CP_UTF16				SQL_DD_CP_UCS2
+
+
 /*
  * IBM extensions for DB2
  */
@@ -267,4 +359,42 @@
 #define SQL_WCHARTYPE_NOCONVERT      0
 #define SQL_WCHARTYPE_DEFAULT        SQL_WCHARTYPE_NOCONVERT
 
+
+/******************* Virtuoso ODBC Extensions *******************/
+
+/*
+ *  ODBC extensions for SQLGetDescField
+ */
+#define SQL_DESC_COL_DV_TYPE		1057L
+#define SQL_DESC_COL_DT_DT_TYPE		1058L
+#define SQL_DESC_COL_LITERAL_ATTR	1059L
+#define SQL_DESC_COL_BOX_FLAGS		1060L
+#define SQL_DESC_COL_LITERAL_LANG	1061L
+#define SQL_DESC_COL_LITERAL_TYPE	1062L
+
+
+/*
+ *  Virtuoso - ODBC SQL_DESC_COL_DV_TYPE
+ */
+#define VIRTUOSO_DV_DATE		129
+#define VIRTUOSO_DV_DATETIME		211
+#define VIRTUOSO_DV_DOUBLE_FLOAT	191
+#define VIRTUOSO_DV_IRI_ID		243
+#define VIRTUOSO_DV_LONG_INT		189
+#define VIRTUOSO_DV_NUMERIC		219
+#define VIRTUOSO_DV_RDF			246
+#define VIRTUOSO_DV_SINGLE_FLOAT	190
+#define VIRTUOSO_DV_STRING		182
+#define VIRTUOSO_DV_TIME		210
+#define VIRTUOSO_DV_TIMESTAMP		128
+#define VIRTUOSO_DV_TIMESTAMP_OBJ	208
+
+
+/*
+ *  Virtuoso - ODBC SQL_DESC_COL_DT_DT_TYPE
+ */
+#define VIRTUOSO_DT_TYPE_DATETIME	1
+#define VIRTUOSO_DT_TYPE_DATE		2
+#define VIRTUOSO_DT_TYPE_TIME		3
+
 #endif /* _IODBCEXT_H */
