@@ -7,8 +7,8 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1995 by Ke Jin <kejin@empress.com>
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1995 Ke Jin <kejin@empress.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -85,7 +85,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include <unicode.h>
+#include "unicode.h"
 
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -196,7 +196,8 @@ _iodbcdm_cfg_parse_str_Internal (PCONFIG pconfig, char *str)
  *  Initialize a configuration from string
  */
 int
-_iodbcdm_cfg_init_str (PCONFIG *ppconf, void *str, int size, int wide)
+_iodbcdm_cfg_init_str (PCONFIG *ppconf, void *str, int size, int wide,
+	DM_CONV *conv)
 {
   PCONFIG pconfig;
 
@@ -207,7 +208,7 @@ _iodbcdm_cfg_init_str (PCONFIG *ppconf, void *str, int size, int wide)
     return -1;
 
   /* parse */
-  if (_iodbcdm_cfg_parse_str (pconfig, str, size, wide) == -1)
+  if (_iodbcdm_cfg_parse_str (pconfig, str, size, wide, conv) == -1)
     {
       _iodbcdm_cfg_done (pconfig);
       return -1;
@@ -223,12 +224,13 @@ _iodbcdm_cfg_init_str (PCONFIG *ppconf, void *str, int size, int wide)
  *  Parse a configuration from string
  */
 int
-_iodbcdm_cfg_parse_str (PCONFIG pconfig, void *str, int size, int wide)
+_iodbcdm_cfg_parse_str (PCONFIG pconfig, void *str, int size, int wide,
+	DM_CONV *conv)
 {
   int ret;
   char *_str;
 
-  _str = wide ? (char *) dm_SQL_WtoU8 (str, size) : str;
+  _str = wide ? (char *) DM_WtoU8 (conv, str, size) : str;
 
   ret = _iodbcdm_cfg_parse_str_Internal (pconfig, _str);
 
