@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -103,5 +103,5 @@ trace_SQLBindParameter (int trace_leave, int retcode,
   _trace_smallint (DecimalDigits);
   _trace_pointer (ParameterValuePtr);
   _trace_len (BufferLength);
-  _trace_len_p (Strlen_or_IndPtr, TRACE_OUTPUT_SUCCESS);
+  _trace_len_p (Strlen_or_IndPtr, 0);    /* just display pointer */
 }
