@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -75,9 +75,7 @@
  */
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <iodbc.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -96,11 +94,11 @@
 #include <sqlext.h>
 #include <sqlucode.h>
 
+#include "unicode.h"
 #include "herr.h"
 #include "henv.h"
 #include "ithread.h"
 #include "trace.h"
-#include "unicode.h"
 
 #define NO_CARBON 1
 #if defined(macintosh)
@@ -186,7 +184,7 @@ trace_set_filename (char *template)
   struct passwd *pwd;
   char *buf;
   size_t buf_len, buf_pos;
-  char tmp[255];
+  char tmp[4096];
 
   /* Make copy of template */
   if (template)
@@ -429,15 +427,9 @@ trace_start(void)
       trace_set_appname ("{No Application Name}");
   }
 #elif defined(__APPLE__)
-#ifdef MACOSX102
-  {
-    trace_set_appname ("{No Application Name}");
-  }
-#else
   {
     trace_set_appname ((char *) getprogname ());
   }
-#endif
 #endif
 
   /*
@@ -717,7 +709,7 @@ _trace_print_function (int func, int trace_leave, int retcode)
       tv.tv_sec--;
       tv.tv_usec += 1000000L;
     }
-  trace_emit ("\n[%06ld.%06ld]\n", tv.tv_sec, tv.tv_usec);
+  trace_emit ("\n[%06ld.%06ld]\n", (long) tv.tv_sec, (long) tv.tv_usec);
 #else
   trace_emit ("\n");
 #endif
@@ -1316,7 +1308,7 @@ _trace_bufferlen (SQLINTEGER length)
     }
 
   if (ptr)
-    trace_emit ("\t\t%-15.15s * %ld (%s)\n", "SQLINTEGER", length, ptr);
+    trace_emit ("\t\t%-15.15s * %ld (%s)\n", "SQLINTEGER", (long) length, ptr);
   else
     trace_emit ("\t\t%-15.15s * %ld\n", "SQLINTEGER", (long) length);
 }
