@@ -5,7 +5,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -73,6 +73,7 @@
  */
 
 #include <iodbc.h>
+#include <iodbcext.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
@@ -123,6 +124,9 @@ adddrivers_to_list (GtkWidget *widget, GtkWidget *dlg)
       goto end;
     }
 
+  SQLSetEnvAttr (henv, SQL_ATTR_APP_UNICODE_TYPE,
+      (SQLPOINTER) SQL_DM_CP_DEF, SQL_IS_UINTEGER);
+
   /* Set the version ODBC API to use */
   SQLSetEnvAttr (henv, SQL_ATTR_ODBC_VERSION, (SQLPOINTER) SQL_OV_ODBC3,
       SQL_IS_INTEGER);
