@@ -5,7 +5,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -73,6 +73,7 @@
  */
 
 #include <iodbc.h>
+#include <iodbcext.h>
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -447,6 +448,9 @@ test_driver_connect (TDSNCHOOSER *choose_t, char *connstr)
       return FALSE;
     }
 
+  SQLSetEnvAttr (henv, SQL_ATTR_APP_UNICODE_TYPE,
+      (SQLPOINTER) SQL_DM_CP_DEF, SQL_IS_UINTEGER);
+
 #if (ODBCVER < 0x300)
   if (SQLAllocConnect (henv, &hdbc) != SQL_SUCCESS)
 #else
