@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -76,7 +76,7 @@
 
 #include <iodbc.h>
 #include <odbcinst.h>
-#include <unicode.h>
+#include "unicode.h"
 
 #include "inifile.h"
 #include "iodbc_error.h"
@@ -187,7 +187,7 @@ SQLReadFileDSNW (LPCWSTR lpszFileName, LPCWSTR lpszAppName,
 
   if (retcode == TRUE)
     {
-      dm_StrCopyOut2_U8toW (_string_u8, lpszString, cbString, pcbString);
+      dm_StrCopyOut2_U8toW ((SQLCHAR *)_string_u8, lpszString, cbString, pcbString);
     }
 
 done:
