@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -77,45 +77,90 @@
 #ifndef _UNICODE_H
 #define _UNICODE_H
 
-
-#if defined (__APPLE__) && !defined (MACOSX102) && !defined (HAVE_CONFIG_H)
-#define HAVE_WCHAR_H 
-#define HAVE_WCSLEN 
-#define HAVE_WCSCPY 
-#define HAVE_WCSNCPY
-#define HAVE_WCSCHR
-#define HAVE_WCSCAT
-#define HAVE_WCSCMP
-#define HAVE_TOWLOWER
-#endif
+#include <iodbc.h>
+#include <iodbcext.h>
 
 #if defined (HAVE_WCHAR_H)
 #include <wchar.h>
 #endif
 
+typedef unsigned char  utf8_t;
+typedef unsigned short ucs2_t;
+typedef unsigned int   ucs4_t;
+
+typedef enum
+  {
+    CP_UCS4    = SQL_DM_CP_UCS4,
+    CP_UTF16   = SQL_DM_CP_UTF16,
+    CP_UTF8    = SQL_DM_CP_UTF8
+  }
+IODBC_CHARSET;
+
+#if defined(SIZEOF_WCHAR)
+
+# if (SIZEOF_WCHAR == 2)
+#  define CP_DEF  CP_UTF16
+# else
+#  define CP_DEF  CP_UCS4
+# endif
+
+#else
+
+#  define CP_DEF  CP_UCS4
+
+#endif
+
+
+
+#define WCHAR_MAXSIZE  4
+
+
+typedef struct _dm_conv
+  {
+    IODBC_CHARSET dm_cp;
+    IODBC_CHARSET drv_cp;
+  }
+DM_CONV;
+
+
+typedef enum
+  {
+    CD_NONE		= 0,
+    CD_A2W		= 1,
+    CD_W2A		= 2,
+    CD_W2W		= 3
+  }
+CONV_DIRECT;
+
+
 
 /*
  *  Max length of a UTF-8 encoded character sequence
  */
-#define UTF8_MAX_CHAR_LEN 4
+#define UTF8_MAX_CHAR_LEN	4	/* According to RFC3629 */
+
+#ifndef MB_CUR_MAX
+#define MB_CUR_MAX		UTF8_MAX_CHAR_LEN
+#endif
+
 
 /*
  *  Function Prototypes
  */
-SQLCHAR *dm_SQL_W2A (SQLWCHAR * inStr, ssize_t size);
-SQLCHAR *dm_SQL_WtoU8 (SQLWCHAR * inStr, ssize_t size);
+SQLCHAR *dm_SQL_W2A (SQLWCHAR * inStr, int size);
+SQLCHAR *dm_SQL_WtoU8 (SQLWCHAR * inStr, int size);
 SQLCHAR *dm_strcpy_W2A (SQLCHAR * destStr, SQLWCHAR * sourStr);
 
-SQLWCHAR *dm_SQL_A2W (SQLCHAR * inStr, ssize_t size);
-SQLWCHAR *dm_SQL_U8toW (SQLCHAR * inStr, SQLSMALLINT size);
+SQLWCHAR *dm_SQL_A2W (SQLCHAR * inStr, int size);
+SQLWCHAR *dm_SQL_U8toW (SQLCHAR * inStr, int size);
 SQLWCHAR *dm_strcpy_A2W (SQLWCHAR * destStr, SQLCHAR * sourStr);
 
 int dm_StrCopyOut2_A2W (SQLCHAR * inStr, SQLWCHAR * outStr, SQLSMALLINT size,
-    SQLSMALLINT * result);
-int dm_StrCopyOut2_U8toW (SQLCHAR * inStr, SQLWCHAR * outStr, size_t size,
-    u_short * result);
+    WORD * result);
+int dm_StrCopyOut2_U8toW (SQLCHAR * inStr, SQLWCHAR * outStr, int size,
+    WORD * result);
 int dm_StrCopyOut2_W2A (SQLWCHAR * inStr, SQLCHAR * outStr, SQLSMALLINT size,
-    SQLSMALLINT * result);
+    WORD * result);
 
 
 # ifdef WIN32
@@ -130,6 +175,63 @@ int dm_StrCopyOut2_W2A (SQLWCHAR * inStr, SQLCHAR * outStr, SQLSMALLINT size,
 #define OPL_A2W(XA, XW, SIZE)      mbstowcs((wchar_t *) XW, (char *) XA, SIZE)
 # endif
 
+int dm_conv_W2W(void *inStr, int len, void *outStr, int size,
+	IODBC_CHARSET icharset, IODBC_CHARSET ocharset);
+int dm_conv_W2A(void *inStr, int inLen, char *outStr, int size,
+	IODBC_CHARSET charset);
+int dm_conv_A2W(char *inStr, int inLen, void *outStr, int size,
+	IODBC_CHARSET charset);
+
+void DM_strcpy_U8toW (DM_CONV *conv, void *dest, SQLCHAR *sour);
+
+size_t DRV_WCHARSIZE(DM_CONV *conv);
+size_t DM_WCHARSIZE(DM_CONV *conv);
+size_t DRV_WCHARSIZE_ALLOC(DM_CONV *conv);
+size_t DM_WCHARSIZE_ALLOC(DM_CONV *conv);
+
+void *DM_A2W(DM_CONV *conv, SQLCHAR * inStr, int size);
+SQLCHAR *DM_W2A(DM_CONV *conv, void * inStr, int size);
+SQLCHAR *DRV_W2A(DM_CONV *conv, void * inStr, int size);
+
+void DM_SetWCharAt(DM_CONV *conv, void *str, int pos, int ch);
+void DRV_SetWCharAt(DM_CONV *conv, void *str, int pos, int ch);
+SQLWCHAR DM_GetWCharAt(DM_CONV *conv, void *str, int pos);
+
+void *DM_WCSCPY(DM_CONV *conv, void *dest, void *sour);
+void *DM_WCSNCPY(DM_CONV *conv, void *dest, void *sour, size_t count);
+void *DRV_WCSNCPY(DM_CONV *conv, void *dest, void *sour, size_t count);
+
+size_t DM_WCSLEN(DM_CONV *conv, void *str);
+size_t DRV_WCSLEN(DM_CONV *conv, void *str);
+
+SQLCHAR *DM_WtoU8(DM_CONV *conv, void *inStr, int size);
+SQLCHAR *DRV_WtoU8(DM_CONV *conv, void *inStr, int size);
+void *DM_U8toW(DM_CONV *conv, SQLCHAR *inStr, int size);
+
+int dm_StrCopyOut2_A2W_d2m (DM_CONV *conv, SQLCHAR *inStr,
+		void *outStr, int size, SQLSMALLINT *result, int *copied);
+int dm_StrCopyOut2_W2A_d2m (DM_CONV *conv, void *inStr,
+		SQLCHAR *outStr, int size, SQLSMALLINT *result, int *copied);
+int dm_StrCopyOut2_U8toW_d2m (DM_CONV *conv, SQLCHAR *inStr,
+		void *outStr, int size, SQLSMALLINT *result, int *copied);
+int dm_StrCopyOut2_W2W_d2m (DM_CONV *conv, void *inStr,
+		void *outStr, int size, SQLSMALLINT *result, int *copied);
+
+int dm_StrCopyOut2_W2A_m2d (DM_CONV *conv, void *inStr,
+		SQLCHAR *outStr, int size, SQLSMALLINT *result, int *copied);
+int dm_StrCopyOut2_W2W_m2d (DM_CONV *conv, void *inStr,
+		void *outStr, int size, SQLSMALLINT *result, int *copied);
+
+
+void *conv_text_d2m(DM_CONV *conv, void *inStr, int size,
+	CONV_DIRECT direct);
+void *conv_text_m2d(DM_CONV *conv, void *inStr, int size,
+	CONV_DIRECT direct);
+
+void * conv_text_m2d_W2W(DM_CONV *conv, void *inStr, SQLLEN size,
+        SQLLEN *copied);
+
+
 
 /*
  *  Replacement functions
